<?php

declare(strict_types=1);

namespace Drupal\inactive_user_management\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\State\StateInterface;
use Drupal\inactive_user_management\Settings\NotificationSettings;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The InactiveUserNotificationSettingsForm form.
 */
class InactiveUserNotificationSettingsForm extends ConfigFormBase {

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames(): array {
    return ['inactive_user_management.notification_settings'];
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'inactive_user_management_settings_form';
  }

  /**
   * The State object.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $drupalState;

  /**
   * The Messenger object.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The renderer object.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected RendererInterface $renderer;

  /**
   * The module handler object.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * The entity type manager object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected TypedConfigManagerInterface $typedConfigManager,
    StateInterface $state,
    MessengerInterface $messenger,
    RendererInterface $renderer,
    ModuleHandlerInterface $module_handler,
    EntityTypeManagerInterface $entity_type_manager,
  ) {
    parent::__construct($config_factory, $typedConfigManager);
    $this->drupalState = $state;
    $this->messenger = $messenger;
    $this->renderer = $renderer;
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    /** @var \Drupal\Core\Config\ConfigFactoryInterface $config_factory */
    $config_factory = $container->get('config.factory');
    /** @var \Drupal\Core\Config\TypedConfigManager $config_typed */
    $config_typed = $container->get('config.typed');
    /** @var \Drupal\Core\State\StateInterface $state */
    $state = $container->get('state');
    /** @var \Drupal\Core\Messenger\MessengerInterface $messenger */
    $messenger = $container->get('messenger');
    /** @var \Drupal\Core\Render\RendererInterface $renderer */
    $renderer = $container->get('renderer');
    /** @var \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler */
    $moduleHandler = $container->get('module_handler');
    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager */
    $entity_type_manager = $container->get('entity_type.manager');
    return new static(
      $config_factory,
      $config_typed,
      $state,
      $messenger,
      $renderer,
      $moduleHandler,
      $entity_type_manager,
    );
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $rendered_token_tree = '';
    if ($this->moduleHandler->moduleExists('token')) {
      $token_tree = [
        '#theme' => 'token_tree_link',
        '#global_types' => TRUE,
      ];
      $rendered_token_tree = $this->renderer->render($token_tree);
    }

    $inputSelector = str_replace('_', '-', NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled');

    $notificationEnabled = [
      'input[data-drupal-selector="edit-' . $inputSelector . '"]' => [
        'checked' => TRUE,
      ],
    ];

    $form[NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Activate or deactivate notification'),
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled') : FALSE,
    ];

    $form['notification_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Inactive User Notification Settings'),
      '#states' => [
        'visible' => $notificationEnabled,
      ],
    ];

    $roles = $this->getRoles();
    if (!empty($roles)) {
      $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles'] = [
        '#type' => 'checkboxes',
        '#title' => $this->t('Notify these roles by email'),
        '#description' => $this->t('All users with these checked roles will receive a notification Email when selected.'),
        '#options' => $roles,
        '#states' => [
          'visible' => [
            [':input[name="type"]' => ['value' => 'role']],
            [':input[name="type"]' => ['value' => 'both']],
          ],
        ],
        '#required' => TRUE,
        '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles') : ['administrator' => 'administrator'],
      ];
    }

    $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Inactivity refers to the access/login field'),
      '#options' => [
        'access' => $this->t('Last access'),
        'login' => $this->t('Last login'),
      ],
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field') : 'access',
      '#description' => $this->t('Should the last access or the last login be checked?'),
    ];

    $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan'] = [
      '#type' => 'select',
      '#title' => $this->t('Period of inactivity'),
      '#options' => [
        3024000 => $this->t('1 month'),
        7862400 => $this->t('3 months'),
        15724800 => $this->t('6 months'),
        23587200 => $this->t('9 months'),
        31536000 => $this->t('12 months/1 year'),
      ],
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan') : '15724800',
      '#description' => $this->t('After the given time a notification will be sent. x months after the last login/access.'),
    ];

    $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification'] = [
      '#type' => 'select',
      '#title' => $this->t('Period between the notifications'),
      '#options' => [
        3024000 => $this->t('1 month'),
        7862400 => $this->t('3 months'),
        15724800 => $this->t('6 months'),
        23587200 => $this->t('9 months'),
        31536000 => $this->t('12 months/1 year'),
      ],
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification') : '15724800',
      '#description' => $this->t('If a notification has already been sent, another notification is sent after the specified time.
      This means x months after the last notification. (If the user was still inactive for longer than specified under ‘Period of inactivity’).'),
    ];

    $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Email subject'),
      '#required' => TRUE,
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject') : 'We miss you!',
    ];

    $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Email body'),
      '#required' => TRUE,
      '#default_value' => $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content') ? $this->drupalState->get(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content') : 'Visit us again soon! We have new interesting content.',
    ];

    if ($this->moduleHandler->moduleExists('token')) {
      $form['notification_settings'][NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content']['#description'] =
        $this->t('This field supports tokens. @browse_tokens_link', ['@browse_tokens_link' => $rendered_token_tree]);
    }

    // Submit.
    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * Gets all the roles from the storage.
   *
   * @return array
   *   Returns an array with all roles except the 'anonymous' and 'authenticated' roles.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getRoles(): array {
    $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
    $roleLabels = [];
    if ($roles) {
      foreach ($roles as $role) {
        if ($role->id() != 'anonymous' && $role->id() != 'authenticated') {
          $roleLabels[$role->id()] = $role->label();
        }
      }
    }
    return $roleLabels;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject'));
    $this->drupalState->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content', $form_state->getValue(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content'));

    $this->messenger->addMessage($this->t('Saved settings.'));
  }

}
