<?php

namespace Drupal\inactive_user_management\Notification;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\inactive_user_management\Settings\GetNotificationSettings;
use Drupal\symfony_mailer\EmailFactoryInterface;

/**
 * The InactiveUserNotification service class.
 */
class InactiveUserNotification {

  /**
   * The EmailFactory service object.
   *
   * @var \Drupal\symfony_mailer\EmailFactoryInterface
   */
  private EmailFactoryInterface $emailFactory;

  /**
   * The EntityTypeManager service object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * The LoggerChannelFactory service object.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  private LoggerChannelFactoryInterface $loggerFactory;

  /**
   * The Time service object.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  private TimeInterface $time;

  /**
   * The GetNotificationSettings service object.
   *
   * @var \Drupal\inactive_user_management\Settings\GetNotificationSettings
   */
  private GetNotificationSettings $notificationSettings;

  /**
   * The InactiveUserNotification constructor.
   */
  public function __construct(
    EmailFactoryInterface $email_factory,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerchannelFactoryInterface $logger_factory,
    TimeInterface $time,
    GetNotificationSettings $get_notification_settings,
  ) {
    $this->emailFactory = $email_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
    $this->time = $time;
    $this->notificationSettings = $get_notification_settings;
  }

  /**
   * Sends an email notification to users who weren't active for a given time.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function sendNotificationToInactiveUsers(): void {
    if ($this->notificationSettings->getState()) {
      $users = $this->getAllUsersToNotify();

      if (!empty($users)) {
        $settings = $this->notificationSettings->getAllNotificationSettings();
        /** @var \Drupal\user\UserInterface $user */
        foreach ($users as $user) {
          $email = $this->emailFactory->sendTypedEmail('inactive_user_mailer', 'notification', $user->getEmail(), $settings['subject'], $settings['content']);
          if (!$email->getError()) {
            $user->set('field_last_notification', $this->time->getCurrentTime())
              ->save();
            $this->loggerFactory->get('Inactive User Management')
              ->info("There was a notification sent to the inactive user: " . $user->getEmail());
          }
        }
      }
    }
  }

  /**
   * Gets a list of all users who need to be notified, based on the settings.
   *
   * @return array|null
   *   Returns all users as array, or null if no user need to be notified.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getAllUsersToNotify(): array|NULL {
    $settings = $this->notificationSettings->getAllNotificationSettings();
    $lastNotification = $this->time->getCurrentTime() - $settings['timespan_notification'];
    $lastActive = $this->time->getCurrentTime() - $settings['timespan'];

    $userQuery = $this->entityTypeManager->getStorage('user')->getQuery();

    $group = $userQuery
      ->orConditionGroup()
      ->condition('field_last_notification', $lastNotification, '<')
      ->notExists('field_last_notification');

    $userIds = $userQuery
      ->accessCheck(FALSE)
      ->condition('status', 1)
      ->condition('roles', array_values($settings['roles']), 'IN')
      ->condition($settings['user_field'], $lastActive, '<')
      ->condition($group)
      ->execute();

    $users = $this->entityTypeManager->getStorage('user')->loadMultiple($userIds);

    if (!empty($users)) {
      return $users;
    }
    return NULL;
  }

}
