<?php

declare(strict_types=1);

namespace Drupal\Tests\inactive_user_management\Kernel;

use Drupal\Core\State\StateInterface;
use Drupal\inactive_user_management\Settings\GetNotificationSettings;
use Drupal\inactive_user_management\Settings\NotificationSettings;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests the GetNotificationSettings service class.
 *
 * @package inactive_user_management
 * @coversDefaultClass \Drupal\inactive_user_management\Settings\GetNotificationSettings
 * @group Drupal\Tests\inactive_user_management\Kernel
 */
class GetNotificationSettingsTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->state = $this->container->get('state');
    $this->settingsService = new GetNotificationSettings(
      $this->state
    );

    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_enabled', 1);
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_user_field', 'access');
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_roles', ['external_user' => 'external_user']);
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan', '15724800');
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_timespan_last_notification', '15724800');
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_subject', 'We miss you!');
    $this->state->set(NotificationSettings::NOTIFICATION_SETTINGS_PREFIX . '_content', 'Hello user/r/n/r/nHow do you feel today?');
  }

  /**
   * The state object.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected StateInterface $state;

  /**
   * The GetNotificationSettings object.
   *
   * @var \Drupal\inactive_user_management\Settings\GetNotificationSettings
   */
  protected GetNotificationSettings $settingsService;

  /**
   * Tests the getAllNotificationSettings method.
   *
   * @covers ::getAllNotificationSettings
   */
  public function testGetAllNotificationSettings(): void {
    $settings = $this->settingsService->getAllNotificationSettings();

    $this->assertEquals(1, $settings['enabled']);
    $this->assertEquals([0 => 'external_user'], $settings['roles']);
    $this->assertEquals('access', $settings['user_field']);
    $this->assertEquals('15724800', $settings['timespan']);
    $this->assertEquals('15724800', $settings['timespan_notification']);
    $this->assertEquals('We miss you!', $settings['subject']);
    $this->assertEquals('Hello user/r/n/r/nHow do you feel today?', $settings['content']);
  }

  /**
   * Tests the getState method.
   *
   * @covers ::getState
   */
  public function testGetState(): void {
    $notificationState = $this->settingsService->getState();
    $this->assertEquals(1, $notificationState);
  }

  /**
   * Tests the getRoles method.
   *
   * @covers ::getRoles
   */
  public function testGetRoles() {
    $roles = $this->settingsService->getRoles();
    $expectedRoles = [0 => 'external_user'];
    $this->assertEquals($expectedRoles, $roles);
  }

  /**
   * Tests the getUserField method.
   *
   * @covers ::getUserField
   */
  public function testGetUserField() {
    $userField = $this->settingsService->getUserField();
    $this->assertEquals('access', $userField);
  }

  /**
   * Tests the getTimespan method.
   *
   * @covers ::getTimespan
   */
  public function testGetTimespan() {
    $timespan = $this->settingsService->getTimespan();
    $this->assertEquals('15724800', $timespan);
  }

  /**
   * Tests the getTimespanBetweenNotifications method.
   *
   * @covers ::getTimespanBetweenNotifications
   */
  public function testGetTimespanBetweenNotifications() {
    $timespan = $this->settingsService->getTimespanBetweenNotifications();
    $this->assertEquals('15724800', $timespan);
  }

  /**
   * Tests the getSubject method.
   *
   * @covers ::getSubject
   */
  public function testGetSubject() {
    $subject = $this->settingsService->getSubject();
    $this->assertEquals('We miss you!', $subject);
  }

  /**
   * Tests the getContent method.
   *
   * @covers ::getContent
   */
  public function testGetContent() {
    $content = $this->settingsService->getContent();
    $this->assertEquals('Hello user/r/n/r/nHow do you feel today?', $content);
  }

}
