<?php

namespace Drupal\Tests\inactive_user_management\Kernel;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\inactive_user_management\Notification\InactiveUserNotification;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\inactive_user_management\MockTime\TestTime;
use Drupal\user\Entity\User;

/**
 * Tests the InactiveUserNotification service class.
 *
 * @package inactive_user_management
 * @coversDefaultClass \Drupal\inactive_user_management\Notification\InactiveUserNotification
 * @group Drupal\Tests\inactive_user_management\Kernel
 */
class InactiveUserNotificationTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field',
    'user',
    'inactive_user_management',
    'symfony_mailer',
    'ultimate_cron',
  ];

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('user');
    $this->installConfig(['inactive_user_management']);

    $notificationSettingsMock = $this->createMock('\Drupal\inactive_user_management\Settings\GetNotificationSettings');
    $notificationSettingsMock->method('getAllNotificationSettings')->willReturn([
      'enabled' => 1,
      'roles' => ['external_user'],
      'user_field' => 'access',
      'timespan' => '3024000',
      'timespan_notification' => '7862400',
      'subject' => 'Default Subject',
      'content' => 'Default Content',
    ]);
    $notificationSettingsMock->method('getState')->willReturn(1);

    $emailMock = $this->createMock('\Drupal\symfony_mailer\Email');
    $emailMock->method('getError')->willReturn(NULL);
    $emailFactoryMock = $this->createMock('\Drupal\symfony_mailer\EmailFactory');
    $emailFactoryMock->method('sendTypedEmail')->willReturn($emailMock);

    $this->entityTypeManager = $this->container->get('entity_type.manager');

    $this->inactiveUserNotification = new InactiveUserNotification(
      $emailFactoryMock,
      $this->entityTypeManager,
      $this->container->get('logger.factory'),
      new TestTime(),
      $notificationSettingsMock
    );

  }

  /**
   * The InactiveUserNotification object.
   *
   * @var \Drupal\inactive_user_management\Notification\InactiveUserNotification
   */
  protected InactiveUserNotification $inactiveUserNotification;

  /**
   * The EntityTypeManager object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Tests the sendNotificationToInactiveUsers method.
   *
   * @covers ::sendNotificationToInactiveUsers
   */
  public function testSendNotificationToInactiveUsers(): void {
    $user1 = User::create([
      'name' => 'user1',
      'status' => 1,
      'roles' => ['external_user'],
    ]);
    $user1->save();
    $this->inactiveUserNotification->sendNotificationToInactiveUsers();
    /** @var \Drupal\user\Entity\User $user */
    $user = $this->entityTypeManager->getStorage('user')->load($user1->id());
    $this->assertTrue($user->hasField('field_last_notification'));
  }

  /**
   * Tests the getAllUsersToNotify method.
   *
   * @covers ::getAllUsersToNotify
   */
  public function testGetAllUsersToNotify(): void {
    // Create test users.
    $user1 = User::create([
      'name' => 'user1',
      'status' => 1,
      'roles' => ['administrator'],
    ]);
    $user1->save();

    // 1735807372 stands for Thu Jan 02 2025 08:42:52 GMT+0000
    $user2 = User::create([
      'name' => 'user2',
      'status' => 1,
      'roles' => ['external_user'],
      'field_last_notification' => 1735807372,
    ]);
    $user2->save();

    // 1727941372 stands for Thu Oct 03 2024 07:42:52 GMT+0000
    $user3 = User::create([
      'name' => 'user3',
      'status' => 1,
      'roles' => ['external_user', 'other_role'],
      'field_last_notification' => 1727941372,
    ]);
    $user3->save();

    // 1727941372 stands for Thu Oct 03 2024 07:42:52 GMT+0000
    $user4 = User::create([
      'name' => 'user4',
      'status' => 0,
      'roles' => ['external_user', 'other_role'],
      'field_last_notification' => 1727941372,
    ]);
    $user4->save();

    $users = $this->inactiveUserNotification->getAllUsersToNotify();

    $this->assertArrayHasKey($user3->id(), $users, 'The user3 was not in array.');

    $user3->set('status', 0);
    $user3->save();
    $this->assertNull($this->inactiveUserNotification->getAllUsersToNotify());
  }

}
